-- This is a player context script for MOO2 1.50 patch project.
--
-- This script shows you some general information about the game.
-- This includes certain starting settings and mods of the currently loaded savestate.
--
-- To run the script press 0 in main screen (the file should be named MAIN0.LUA)

RANDOM_OPPONENTS_ID = 9

local function difficulty_int_to_name(difficulty)
	local string difficulty_string = "UNKNOWN"
	if difficulty == 0 then
		difficulty_string = "Tutor"
	elseif difficulty == 1 then
		difficulty_string = "Easy"
	elseif difficulty == 2 then
		difficulty_string = "Average"
	elseif difficulty == 3 then
		difficulty_string = "Hard"
	elseif difficulty == 4 then
		difficulty_string = "Impossible"
	end
	return difficulty_string .. " (" .. (difficulty + 1) .. "/5)"
end

local function galaxy_size_int_to_name(size, is_cluster)
	local string size_string = "UNKNOWN"
	if size == 0 then
		size_string = "Small"
	elseif size == 1 then
		size_string = "Medium"
	elseif size == 2 then
		size_string = "Large"
	elseif size == 3 then
		if is_cluster == 1 then
			size_string = "Cluster"
		else
			-- huge is originally encoded with same size as cluster
			size = size + 1
			size_string = "Huge"
		end
	end
	return size_string .. " (" .. (size + 1) .. "/5)"
end

local function tech_level_int_to_name(level, is_postwarp)
	local string level_string = "UNKNOWN"
	if level == 0 then
		level_string = "Pre-warp"
	elseif level == 1 then
		if is_postwarp == 1 then
		    level = level + 1
			level_string = "Post-warp"
		else
			level_string = "Average"
		end
	elseif level == 2 then
		level = level + 1
		level_string = "Advanced"
	end
	return level_string .. " (" .. (level + 1) .. "/4)"
end

-- note: lua's string.gsub, string.gmatch and string.find methods are not supported
local function extract_mod_names(str)
	mod_names= ""
	if str == nil then
		return "None."
	end
	last_backslash_index=1
	for i = 1, #str do
		if str:sub(i,i) == '\\' then
			last_backslash_index = i
		elseif str:sub(i,i) == ";" then
		    mod_names = mod_names .. "\n" .. string.sub(str, last_backslash_index + 1, i - 5)
		    last_backslash_index = i
		end
	end
	if last_backslash_index > 1 then
		mod_names = mod_names .. "\n" .. string.sub(str, last_backslash_index + 1, - 5)
	end
	return mod_names
end

local function nr_players_output(num_players)
	output = "UNKNOWN"
	if game.num_players == RANDOM_OPPONENTS_ID then
		output = "Random"
	else
		output = tostring(num_players)
	end
	return output
end

game = get_game()

output = "Game Information\n"
output = output .. "\n" .. "Number of Players: " .. nr_players_output(game.num_players)
output = output .. "\n" .. "Difficulty: " .. difficulty_int_to_name(game.difficulty)
output = output .. "\n" .. "Galaxy Size: " .. galaxy_size_int_to_name(game.galaxy_size, game.is_cluster)
output = output .. "\n" .. "Tech Level: " .. tech_level_int_to_name(game.civ, game.is_postwarp)
output = output .. "\n"
output = output .. "\n" .. "MOO2-Version started with: " .. game.version_started
output = output .. "\n" .. "MOO2-Version saved with:   " .. game.version_saved
output = output .. "\n"
output = output .. "\n" .. "Core mod: " .. tostring(get_conf("mod_name")) .. " (" .. tostring(get_conf("mod_id")) .. ")"
output = output .. "\n"
output = output .. "\n" .. "Pregame scripts: " .. tostring(extract_mod_names(get_conf("pregame_script")))
output = output .. "\n"
output = output .. "\n" .. "Postgame scripts: " .. tostring(extract_mod_names(get_conf("newgame_postprocessor_script")))

msgbox(output)