-- This is a player context script for MoO2 1.50 patch project.
--
-- This script shows you which techs your opponents have and you don't. It shows
-- only opponents in contact and only if /noreport option is off. It brings up a
-- message box for each opponent who has something.
--
-- To run the script press 1 in main screen (the file should be named MAIN1.LUA)

SORT_TECH_BY_RP_INSTEAD_OF_ALPHABETICALLY = true
SHOW_RP_COST_FOR_INDIVIDUAL_TECH = false

if get_conf("/noreport") == 1 then
    msgbox("/noreport option prevents tech check.")
    return
end

-- note: lua's string.gsub, string.gmatch and string.find methods are not supported
local function format_tech_string(str)
    if str == 'anti_missile_rocket' then
        return 'Anti-Missile Rockets'
    elseif str == 'anti_matter_bomb' then
        return 'Anti-Matter Bomb'
    elseif str == 'anti_matter_drive' then
        return 'Anti-Matter Drive'
    elseif str == 'anti_matter_torpedo' then
        return 'Anti-Matter Torpedo'
    elseif str == 'anti_grav_harness' then
        return 'Anti-Grav Harness'
    elseif str == 'bio_terminator' then
        return 'Bio-Terminator'
    elseif str == 'class_iii_shield' then
        return 'Class III Shield'
    elseif str == 'class_vii_shield' then
        return 'Class VII Shield'
    elseif str == 'cyber_security_link' then
        return 'Cyber-Security Link'
    elseif str == 'hyper_x_capacitors' then
        return 'Hyper-X Capacitors'
    elseif str == 'multi_wave_ecm_jammer' then
        return 'Multi-Wave Ecm Jammer'
    elseif str == 'robo_miners' then
        return 'Robo-Miners'
    elseif str == 'sub_space_communications' then
        return 'Sub-Space Communications'
    end
    local formated_string = str:sub(1,1):upper()
    local next_char_uppercase = false
    for i = 2, #str do
        if str:sub(i,i) == '_' then
            formated_string = formated_string .. ' '
            next_char_uppercase = true
        else
            if next_char_uppercase then
                formated_string = formated_string .. str:sub(i,i):upper()
                next_char_uppercase = false
            else
                formated_string = formated_string .. str:sub(i,i)
            end
        end
    end
    return formated_string
end

local function separate_thousands(integer)
    input_string = tostring(integer)
    output_string = ""
    for i = #input_string, 1, -1 do
        output_string = input_string:sub(i,i) .. output_string
        if (#input_string - i + 1) % 3 == 0 and i ~= 1 then
            output_string = "," .. output_string
        end
    end
    return output_string
end

local function beautify(str, rp_cost)
	if SHOW_RP_COST_FOR_INDIVIDUAL_TECH == true then
		return separate_thousands(rp_cost) .. ' - ' .. format_tech_string(str)
	else
		return format_tech_string(str)
	end
end

local function sort_tech_by_rp(a, b)
    local cost_a = fields[techs[a].field].cost
	local cost_b = fields[techs[b].field].cost
	if cost_a > cost_b then
		return true
	elseif cost_a == cost_b then
		return a < b
	else
		return false
	end
end

c = get_current_player_id()
ct = get_player_techs(c)
techs = rget_techs()
fields = rget_fields()
shown = 0
for i,p in pairs(get_players()) do
    if i ~= c and p ~= nil then
        local out = ""
        local tt = get_player_techs(i)
        local found = {}
        local rp_costs = {}
        if tt ~= nil then
            for t,status in pairs(tt) do
                if status == 3 and ct[t] ~= 3 then
                    table.insert(found, t)
                end
            end
        end
        if #found > 0 then
            local summed_rp_value = 0
            shown = 1
            if SORT_TECH_BY_RP_INSTEAD_OF_ALPHABETICALLY then
                table.sort(found, sort_tech_by_rp)
            else
                table.sort(found)
            end
            local step = 20
            for j = 1, #found, step do
                local o = p.race .. " tech unknown to us:\n\n"
                for k = j, j + step -1 do
                    if k <= #found then
                        rp_cost = fields[techs[found[k]].field].cost
                        o = o .. beautify(found[k], rp_cost) .. "\n"
                        summed_rp_value = summed_rp_value + rp_cost
                    end
                end
                if j / step + 1 == #found / step + 1 or #found <= step then
                    o = o .. "\nTotal: " .. #found .. " techs, worth " .. separate_thousands(summed_rp_value) .. " RP\n"
                end
                if #found > step then
                    o = o .. "\n(page " .. (j / step + 1) .. "/" .. (#found / step + 1) .. ")"
                end
                msgbox(o)
            end
        end
    end
end
if shown == 0 then
    msgbox("No race in contact has tech unknown to us.")
end
